
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//	V.R. CONFERENCING APPLICATION                                       //
//                                                                          //
//	Written by R.M.J.Tucker                                             //
//                                                                          //
//	(C) Copyright 1996                                                  //
//                                                                          //
//	COLOUR.CPP                                                          //
//                                                                          //
//	Implementation of COLOUR class.                                     //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

#include "COLOUR\COLOUR.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Constructor creates and intialises a colour gradation in the palette.    //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

COLOUR :: COLOUR ( int BaseColour, int NumLevels, unsigned IntensityBits, int MaxRed, int MaxGreen, int MaxBlue )

/*	PRE	:	TRUE
	POST	:	Colour gradation setup in palette for BaseColour
*/

{
	Set ( BaseColour, NumLevels, IntensityBits, MaxRed, MaxGreen, MaxBlue );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Alter the colour palette with a gamma corrected gradation.               //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void COLOUR :: Set ( int pBaseColour, int pNumLevels, unsigned pIntensityBits, int pMaxRed, int pMaxGreen, int pMaxBlue )

/*	PRE	:	TRUE
	POST	:	Sets up a palette block for this colour.
*/

{
	int	j;
	union	REGS regset;
	struct	SREGS sregset;
	static	unsigned char PaletteBlock [ 256 ] [ 3 ];

	BaseColour = pBaseColour;
	NumLevels = pNumLevels;
	IntensityBits = pIntensityBits;
	MaxBlue = pMaxBlue;
	MaxGreen = pMaxGreen;
	MaxRed = pMaxRed;

	/*	Gamma-corrected DAC color components for 
		64 linear levels from 0% to 100% intensity	*/

	static unsigned char GammaTable [] = {
		0,  10, 14, 17, 19, 21, 23, 24, 26, 27, 28, 29, 31, 32, 33, 34,
		35, 36, 37, 37, 38, 39, 40, 41, 41, 42, 43, 44, 44, 45, 46, 46,
		47, 48, 48, 49, 49, 50, 51, 51, 52, 52, 53, 53, 54, 54, 55, 55,
		56, 56, 57, 57, 58, 58, 59, 59, 60, 60, 61, 61, 62, 62, 63, 63
	};

	/*	Setup the palette
		block.	*/

	for ( j = 0; j < NumLevels; j++ ) {
		PaletteBlock [ j ] [ 0 ] = ( GammaTable [ ( int ) ( ( ( double ) MaxRed * ( 1.0 - ( double ) j / ( double ) ( NumLevels - 1 ) ) ) + 0.5 ) ] );
		PaletteBlock [ j ] [ 1 ] = ( GammaTable [ ( int ) ( ( ( double ) MaxGreen * ( 1.0 - ( double ) j / ( double ) ( NumLevels - 1 ) ) ) + 0.5 ) ] );
		PaletteBlock [ j ] [ 2 ] = ( GammaTable [ ( int ) ( ( ( double ) MaxBlue * ( 1.0 - ( double ) j / ( double ) ( NumLevels - 1 ) ) ) + 0.5 ) ] );
	}

	/*	Now set up the palette with
		the given palette block.	*/

	regset.x.ax = 0x1012;
	regset.x.bx = BaseColour;
	regset.x.cx = NumLevels;
	regset.x.dx = FP_OFF ( PaletteBlock );
	sregset.es = FP_SEG ( PaletteBlock );
	int86x ( 0x10, &regset, &regset, &sregset );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Operation to return the colour values for this colour.                   //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void COLOUR :: Get ( int *base, int *levels, unsigned *intensity )

/*	PRE	:	TRUE
	POST	:	Returns information about this colour block.
*/

{
    *base = BaseColour;
    *levels = NumLevels;
    *intensity = IntensityBits;
}



