
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//	V.R. CONFERENCING APPLICATION                                       //
//                                                                          //
//	Written by R.M.J.Tucker                                             //
//                                                                          //
//	(C) Copyright 1996                                                  //
//                                                                          //
//	MOUSE.CPP                                                           //
//                                                                          //
//	Implementation of MOUSE class.                                      //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////
	
#include "IO\MOUSE.H"

#include <conio.h>
#include <stdio.h>

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Operation to construct the mouse object.                                 //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

MOUSE :: MOUSE ( void )

/*	PRE	:	TRUE
	POST	:	Initialise the mouse driver and set it to mid screen.
*/

{
	Initialise ();
	SetPosition ( 320, 100 );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Operation to initialise the mouse.                                       //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void MOUSE :: Initialise ( void )

/*	PRE	:	TRUE
	POST	:	Initialises the mouse driver
*/

{	
	union REGS ireg;

	ireg.x.ax = 0;
	int86 ( 0x33, &ireg, &ireg );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Operation to reset the mouse.                                            //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void MOUSE :: Reset ( void )

/*	PRE	:	TRUE
	POST	:	Resets the mouse driver
*/

{
	union REGS ireg;

	ireg.x.ax = 0x21;
	int86 ( 0x33, &ireg, &ireg );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Operation to switch on the mouse pointer.                                //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void MOUSE :: On ( void )

/*	PRE	:	TRUE
	POST	:	Turns on the mouse pointer
*/

{
	union REGS ireg;

	ireg.x.ax = 1;
	int86 ( 0x33, &ireg, &ireg );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Operation to switch off the mouse pointer.                               //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void MOUSE :: Off ( void )

/*	PRE	:	TRUE
	POST	:	Turns off the mouse pointer
*/

{
	union REGS ireg;

	ireg.x.ax = 2;
	int86 ( 0x33, &ireg, &ireg );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Operation return the position and status of the mouse.                   //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

int MOUSE :: GetPosition ( int *x, int *y )
	
/*	PRE	:	TRUE
	POST	:	Returns the position of the mouse and button status
*/

{
	union REGS ireg;

	ireg.x.ax = 3;
	int86 ( 0x33, &ireg, &ireg );

	/*	Return the
		position.	*/

	*x = ireg.x.cx;
	*y = ireg.x.dx;

	/*	Return the button
		press status flag.	*/

	return ( ireg.x.bx );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Operation to set the position of the mouse pointer.                      //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void MOUSE :: SetPosition ( int x, int y )

/*	PRE	:	TRUE
	POST	:	Sets the position of the mouse on screen
*/

{
	union REGS ireg;

	ireg.x.ax = 4;
	ireg.x.cx = x;
	ireg.x.dx = y;
	int86 ( 0x33, &ireg, &ireg );
}

