
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//	V.R. CONFERENCING APPLICATION                                       //
//                                                                          //
//	Written by R.M.J.Tucker                                             //
//                                                                          //
//	(C) Copyright 1996                                                  //
//                                                                          //
//	MAIN.CPP                                                            //
//                                                                          //
//	Implementation of MAIN.                                             //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////
	
#include <conio.h>
#include <bios.h>
#include <stdlib.h>

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Global objects - need mode, bitmap, etc.                                 //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

/*	Setup the graphics
	system.	*/

#include "MODE\MODE.H"

GRAPHICSMODE Mode ( _320by400in256 );

/*	And create some
	colours.	*/

#include "COLOUR\COLOUR.H"

COLOUR Blue	 ( 16, 	32, 5, 0, 0, 0x3F );
COLOUR White	 ( 48, 	32, 5, 0x3F, 0x3F, 0x3F );
COLOUR Grey	 ( 80, 	32, 5, 0x3f, 0x2F, 0x10 );
COLOUR Green	 ( 112, 32, 5, 0x4, 0x20, 0x4 );

/*	And a new type
	`bird' object.	*/

#include "OBJECT\BIRD.H"

BIRD MyBird;

/*	And a ground
	plane.	*/

#include "OBJECT\GROUND.H"

GROUND Ground;

/*	And a geometric
	transformation.	*/

#include "VIEW\VIEW.H"

VIEWPROJECTION CircleView, PanView, TrackView;

#include "CAMERA\PANCAM.H"

PANCAM PanCam;

#include "CAMERA\CIRCLCAM.H"

CIRCLECAM CircleCam;

/*	Setup the
	light source.	*/

CAMERA Light;

/*	Two windows to demonstrate
	the independant views.	*/

#include "VIEW\WINDOW.H"

WINDOW PanWin ( 0, 0, 320, 400 ), CircleWin ( 0, 0, 320, 400 );

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// MAIN TEST HARNESS - Testing cube rotation.                               //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void main ( void )

{                                   
	/*	There is 1 object
		in the scene.	*/

	#define NUMOBJECTS 1

	/*	Set of objects to
		pan around onto.	*/

	OBJECT3D *Objects [NUMOBJECTS] = {
		&MyBird
	};

	/*	Object Pan is currently
		focused on.	*/

	int i;

	/*	Set the initial
		position of my bird.	*/

	MyBird.GetCamera ()->SetLocation ( 50.0, 400.0, 1500.0 );

	/*	Set the initial position
		for the pan-cam.	*/

	PanCam.SetLocation ( 0.0, 550.0, 0.0 );

	/*	Set the focus
		for the pan-cam.	*/

	PanCam.SetFocus ( &MyBird.GetCamera ()->GetLocation (), &MyBird.GetCamera ()->GetLocation (), 80.0 );
	CircleCam.SetFocus ( &MyBird.GetCamera ()->GetLocation (), 150.0, 150.0 );

	/*	Set the location
		of the light.	*/

	Light.SetLocation ( 0.0, 100.0, -1000.0 );

	/*	Tie the camera into the 
		viewing transformation.	*/

	PanView.SetCamera ( &PanCam );
	CircleView.SetCamera ( &CircleCam );

	/*	And tie the light 
		source in also.	*/

	PanView.SetLight ( &Light );
	CircleView.SetLight ( &Light );

	int CurrentOne = 1;

	for ( ;; ) {

		if ( CurrentOne == 0 ) {

		/*	Ready the pan
			camera.	*/

		PanCam.Ready ();

		/*	Reset the view
			for next frame.	*/

		PanView.Reset ( PanWin );

		/*	Show the
			ground.	*/

		Ground.Show ( PanView, Mode );

		/*	Add the objects
			to the view.	*/

		for ( i = 0; i < NUMOBJECTS; i++ ) {
			Objects [i]->Show ( PanView );
		}

		/*	Show the
			view.	*/

		PanView.Show ( Mode );
		} else {


  		/*	Ready the pan
			camera.	*/

		CircleCam.Ready ();

		/*	Reset the view
			for next frame.	*/

		CircleView.Reset ( CircleWin );

		/*	Show the
			ground.	*/

		Ground.Show ( CircleView, Mode );

		/*	Add the objects
			to the view.	*/

		for ( i = 0; i < NUMOBJECTS; i++ ) {
			Objects [i]->Show ( CircleView );
		}

		/*	Show the
			view.	*/

		CircleView.Show ( Mode );
		}
		/*	Place the bitmap
			view onto the screen.	*/

		Mode.OutText ( 16, 300, "GOLDEN BIRD DEMO BY ROBIN M J TUCKER", 0 );
		Mode.OutText ( 8, 316, "HIT SPACE TO SWITCH VIEWS OR X TO QUIT", 0 );

		Mode.FlipPage ();

		/*	Clear the back
			page.	*/

		Mode.Clear ( 16 );

		if ( kbhit () ) {
			char Key = getch ();

			if ( Key == 0 ) {
				getch ();
			}
			if ( Key == 'X' || Key == 'x' ) {
				textmode ( C80 );
				exit ( 0 );
			} else {
				if ( CurrentOne == 1 ) {
					CurrentOne = 0;
				} else {
					CurrentOne = 1;
				}
			}
		}
	}
}

