
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//	V.R. CONFERENCING APPLICATION                                       //
//                                                                          //
//	Written by R.M.J.Tucker                                             //
//                                                                          //
//	(C) Copyright 1996                                                  //
//                                                                          //
//	VECTOR.CPP                                                          //
//                                                                          //
//	Implementation of VECTOR class.                                     //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////
	
#include "MATH\VECTOR.H"

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Create a vector ( empty and undefined ).                                 //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

VECTOR :: VECTOR ( void )

/*	PRE	:	TRUE
	POST	:	TRUE
*/

{

}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Create a vector by giving it initial values.                             //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

VECTOR :: VECTOR ( REAL x, REAL y, REAL z )

/*	PRE	:	TRUE
	POST	:	This = ( x, y, z )
*/

{
	v [0] = x;
	v [1] = y;
	v [2] = z;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Create a vector giving it another vectors values.                        //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

VECTOR :: VECTOR ( VECTOR& Vector )

/*	PRE	:	TRUE
	POST	:	This = Vector
*/

{
	v [0] = Vector.v [0];
	v [1] = Vector.v [1];
	v [2] = Vector.v [2];
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Create a vector by giving it the values of a point.                      //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

VECTOR :: VECTOR ( POINT& Point )

/*	PRE	:	TRUE
	POST	:	This = ( VECTOR ) Point
*/

{
	v [0] = Point.v [0];
	v [1] = Point.v [1];
	v [2] = Point.v [2];
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Create a vector from one point to another.                               //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

VECTOR :: VECTOR ( POINT& From, POINT& To )

/*	PRE	:	TRUE
	POST	:	This = ( VECTOR ) ( To - From )
*/

{
	v [0] = To.v [0] - From.v [0];
	v [1] = To.v [1] - From.v [1];
	v [2] = To.v [2] - From.v [2];
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Create a vector from one point to another.                               //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

VECTOR :: VECTOR ( POINT *From, POINT *To )

/*	PRE	:	TRUE
	POST	:	This = ( VECTOR ) ( To - From )
*/

{
	v [0] = To->v [0] - From->v [0];
	v [1] = To->v [1] - From->v [1];
	v [2] = To->v [2] - From->v [2];
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Create a vector by giving it initial values.                             //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void VECTOR :: Set ( REAL x, REAL y, REAL z )

/*	PRE	:	TRUE
	POST	:	This = ( x, y, z )
*/

{
	v [0] = x;
	v [1] = y;
	v [2] = z;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Create a vector from one point to another.                               //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void VECTOR :: Make ( POINT *From, POINT *To )

/*	PRE	:	TRUE
	POST	:	This = ( VECTOR ) ( To - From )
*/

{
	v [0] = To->v [0] - From->v [0];
	v [1] = To->v [1] - From->v [1];
	v [2] = To->v [2] - From->v [2];
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Cross multiply two vectors together ( this vector and another ).         //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void VECTOR :: Cross ( VECTOR& u )

/*	PRE	:	TRUE
	POST	:	This = ( This [cross product] v )
*/

{       	
	REAL x, y, z;

	x = ( v [1] * u.v [2] ) - ( v [2] * u.v [1] );
	y = ( v [2] * u.v [0] ) - ( v [0] * u.v [2] );
	z = ( v [0] * u.v [1] ) - ( v [1] * u.v [0] );

	v [0] = x;
	v [1] = y;
	v [2] = z;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Return the dot product of this vector and another.                       //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

REAL VECTOR :: Dot ( VECTOR& u )

/*	PRE	:	TRUE
	POST	:	RETURN ( This [dot product] u )
*/

{
	return ( v [0] * u.v [0] + v [1] * u.v [1] + v [2] * u.v [2] );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Return the length of this vector.                                        //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

REAL VECTOR :: Length ( void )

/*	PRE	:	TRUE
	POST	:	RETURNS |This| 
*/

{
	return ( sqrt ( v [0] * v [0] + v [1] * v [1] + v [2] * v [2] ) );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Make this vector unit length by dividing through by length.              //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void VECTOR :: Unit ( void )

/*	PRE	:	TRUE
	POST	:	|This| = 1.0
*/

{
	REAL w;

	w = Length ();
	v [0] /= w;
	v [1] /= w;
	v [2] /= w;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Assign a point to a vector                                               //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

VECTOR& VECTOR :: operator = ( POINT& Point )

/*	PRE	:	TRUE
	POST	:	The vector is made from the point
*/

{
	v [0] = Point.v [0];
	v [1] = Point.v [1];
	v [2] = Point.v [2];

	return ( *this );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Rotate a vector in the XZ plane.                                         //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void VECTOR :: RotateXZ ( REAL Angle )

/*	PRE	:	TRUE
	POST	:	The given vector is rotated
*/

{
	REAL Radians = RADIAN ( Angle ), x, z, Cos, Sin;

	Cos = cos ( Radians );
	Sin = sin ( Radians );

	/*	Rotate 
		explicitly.	*/

	x = v [0] * Cos - v [2] * Sin;
	z = v [0] * Sin + v [2] * Cos;
	v [0] = x;
	v [2] = z;
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Rotate a vector in the YZ plane.                                         //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void VECTOR :: RotateYZ ( REAL Angle )

/*	PRE	:	TRUE
	POST	:	The given vector is rotated
*/

{
	REAL Radians = RADIAN ( Angle ), x, y, Cos, Sin;

	Cos = cos ( Radians );
	Sin = sin ( Radians );

	/*	Rotate 
		explicitly.	*/

	x = v [1] * Cos - v [2] * Sin;
	y = v [1] * Sin + v [2] * Cos;
	v [1] = x;
	v [2] = y;
}