
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//	V.R. CONFERENCING APPLICATION                                       //
//                                                                          //
//	Written by R.M.J.Tucker                                             //
//                                                                          //
//	(C) Copyright 1996                                                  //
//                                                                          //
//	MODETEX.CPP                                                         //
//                                                                          //
//	Implementation of MODE class.                                       //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////
	

#include "MODE\MODE.H"

#include <conio.h>
#include <stdio.h>
#include <math.h>
#include <dos.h>


//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// These are statics to avoid excessive parameter passing on the stack.     //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

	static int	MaxVert, DestY, nVertices;
	static INTPOINT	*VertexPtr;
	static INTPOINT	*TexVertsPtr;

	extern PIXEL 	*_ASM_Ptr;
	extern long 	_ASM_SourceX;
	extern long 	_ASM_SourceY;
	extern PIXEL	*_ASM_Tex;
	extern long	_ASM_StepX;
	extern long	_ASM_StepY;
	extern char	_ASM_TexWidth;
	extern int	_ASM_Length;
	extern PIXEL	_ASM_Colour;

	extern "C" void _ASM_TexScanLine ( void );
	extern "C" void _ASM_TexScanHoleLine ( void );
	extern "C" long _ASM_FixedDiv ( long, long );
	extern "C" long _ASM_FixedMul ( long, long );

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// This function is Abrash's edge stepper for the edges of the polygon.     //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

int GRAPHICSMODE :: TexStepEdge ( EDGESCAN * Edge )

/*	PRE	:
	POST	:
*/

{
	if ( --Edge->RemainingScans == 0 ) {
		if ( TexSetUpEdge ( Edge, Edge->CurrentEnd ) == 0 ) {
			return ( 0 );
		}
		return ( 1 );
	}

	Edge->SourceX += Edge->SourceStepX;
	Edge->SourceY += Edge->SourceStepY;

	Edge->DestX += Edge->DestXIntStep;  /* whole pixel step */

	if ( ( Edge->DestXErrTerm += Edge->DestXAdjUp ) > 0 ) {
		Edge->DestX += Edge->DestXDirection;
		Edge->DestXErrTerm -= Edge->DestXAdjDown;
	}

	return ( 1 );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// This function is Abrash's edge stepper setup for polygon edges.          //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

int GRAPHICSMODE :: TexSetUpEdge ( EDGESCAN * Edge, int StartVert )

/*	PRE	:
	POST	:
*/

{
	int NextVert, DestXWidth;
	long DestYHeight;

	for (;;) {
		if ( StartVert == MaxVert ) {
			return ( 0 );
		}

		NextVert = StartVert + Edge->Direction;
		if ( NextVert >= nVertices ) {
			NextVert = 0;
		} else if ( NextVert < 0 ) {
			NextVert = nVertices - 1;
		}

		if ( ( Edge->RemainingScans = VertexPtr [NextVert].y - VertexPtr [StartVert].y) != 0 ) {
			DestYHeight = INT_TO_FIXED ( Edge->RemainingScans );
			Edge->CurrentEnd = NextVert;
			Edge->SourceX = INT_TO_FIXED ( TexVertsPtr [StartVert].x );
			Edge->SourceY = INT_TO_FIXED ( TexVertsPtr [StartVert].y );
			Edge->SourceStepX = _ASM_FixedDiv ( INT_TO_FIXED ( TexVertsPtr [NextVert].x ) - Edge->SourceX, DestYHeight );
			Edge->SourceStepY = _ASM_FixedDiv (INT_TO_FIXED( TexVertsPtr [NextVert].y ) - Edge->SourceY, DestYHeight );
			Edge->DestX = VertexPtr [StartVert].x;
			if ( ( DestXWidth = ( VertexPtr [NextVert].x - VertexPtr [StartVert].x ) ) < 0 ) {
				Edge->DestXDirection = -1;
				DestXWidth = -DestXWidth;
				Edge->DestXErrTerm = 1 - Edge->RemainingScans;
				Edge->DestXIntStep = -( DestXWidth / Edge->RemainingScans );
			} else {
				Edge->DestXDirection = 1;
				Edge->DestXErrTerm = 0;
				Edge->DestXIntStep = DestXWidth / Edge->RemainingScans;
			}
			Edge->DestXAdjUp = DestXWidth % Edge->RemainingScans;
			Edge->DestXAdjDown = Edge->RemainingScans;
			return ( 1 );
		}
		StartVert = NextVert;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Abrash's outline scanner for the stepped edges of a polygon.             //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void GRAPHICSMODE :: TexScanOutLine ( EDGESCAN * LeftEdge, EDGESCAN * RightEdge )

/*	PRE	:
	POST	:
*/

{
	long		SourceX = LeftEdge->SourceX;
	long		SourceY = LeftEdge->SourceY;
	int		DestX = LeftEdge->DestX;
	int		DestXMax = RightEdge->DestX;
	long   		DestWidth;
	long		SourceXStep, SourceYStep;

	if ( ( DestXMax <= MinY ) || ( DestX >= MaxY ) ) {
		return;
	}

	if ( ( DestXMax - DestX ) <= 0 ) {
		return;
	}

	DestWidth = INT_TO_FIXED ( DestXMax - DestX );


	SourceXStep = _ASM_FixedDiv ( RightEdge->SourceX - SourceX, DestWidth );
	SourceYStep = _ASM_FixedDiv ( RightEdge->SourceY - SourceY, DestWidth );
	SourceX += SourceXStep >> 1;
	SourceY += SourceYStep >> 1;

	if ( DestXMax > MaxY ) {
		DestXMax = MaxY;
	}

	if ( DestX < MinY ) {
		SourceX += _ASM_FixedMul ( SourceXStep, INT_TO_FIXED ( MinY - DestX ) );
		SourceY += _ASM_FixedMul ( SourceYStep, INT_TO_FIXED ( MinY - DestX ) );
		DestX = MinY;
	}

	int Masks [] = { 1, 2, 4, 8 };

	MAPMASK ( Masks [DestY&3] );

	_ASM_Ptr = GETADDRESS ( DestY, DestX );
	_ASM_SourceX = ( SourceX + 0x8000 );
	_ASM_SourceY = ( SourceY + 0x8000 );
	_ASM_StepX = SourceXStep;
	_ASM_StepY = SourceYStep;
	_ASM_Length = ( DestXMax - DestX );
	_ASM_TexScanLine ();

	/*
		{
			PIXEL *Ptr, Pix;

			Ptr = GETADDRESS ( DestY, DestX );
			SourceX += 0x8000;
			SourceY += 0x8000;
			for (; DestX < DestXMax; DestX++ ) {
				Pix = * ( TexMapBits + ( int ) ( SourceX >> 16 ) + ( ( int ) ( SourceY >> 16 ) << TexMapDeltaY ) );
				*Ptr = ( Pix + tColour );
				SourceX += SourceXStep;
				SourceY += SourceYStep;
				Ptr += 80;
			}
		}
		*/
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Render a texture mapped polygon onto the screen.                         //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void GRAPHICSMODE :: TextureMapPolygon ( int nVerts, INTPOINT *Verts, INTPOINT *tVerts, BOOL Culled, PIXEL Colour )

/*	PRE	:	TRUE
	POST	:	Render textured polygon with the given colour.
*/

{
	int		pMinY, pMaxY, MinVert, i, j;
	EDGESCAN	LeftEdge, RightEdge;
	static INTPOINT MyVerts [64], MyTexVerts [64];

	nVertices = nVerts;
	VertexPtr = Verts;
	TexVertsPtr = tVerts;
	_ASM_Tex = Texture->GetPixels ();
	_ASM_TexWidth = Texture->GetDelta ();
	_ASM_Colour = Colour;

	if (nVertices < 3) {
		return;
	}

	if ( Culled == TRUE ) {

		/*	Culled, so vertices
			are in reverse order.	*/

		for ( j = 0, i = ( nVertices - 1 ); i > -1; i--, j++ ) {
			MyVerts [i].x = VertexPtr[j].y;
			MyVerts [i].y = VertexPtr[j].x;
			MyTexVerts [i].x = TexVertsPtr[j].x;
			MyTexVerts [i].y = TexVertsPtr[j].y;
		}
	} else {

		/*	Was not culled so vertices
			remain in the same order.	*/

		for ( i = 0; i < nVertices; i++ ) {
			MyVerts [i].x = VertexPtr[i].y;
			MyVerts [i].y = VertexPtr[i].x;
			MyTexVerts [i].x = TexVertsPtr[i].x;
			MyTexVerts [i].y = TexVertsPtr[i].y;
		}
	}

	TexVertsPtr = MyTexVerts;
        VertexPtr = MyVerts;

	pMinY = 32767;
	pMaxY = -32767;
	for (i = 0; i < nVertices; i++ ) {
		if ( VertexPtr [i].y < pMinY ) {
			pMinY = VertexPtr [i].y;
			MinVert = i;
		}
		if ( VertexPtr [i].y > pMaxY ) {
			pMaxY = VertexPtr [i].y;
			MaxVert = i;
		}
	}

	if ( pMinY >= pMaxY ) {
		return;
	}
	DestY = pMinY;

	LeftEdge.Direction = -1;
	TexSetUpEdge ( &LeftEdge, MinVert );
	RightEdge.Direction = 1;
	TexSetUpEdge ( &RightEdge, MinVert );

	for (;;) {
		if ( DestY >= MaxX ) {
			return;
		}
		if ( DestY >= MinX ) {
			TexScanOutLine ( &LeftEdge, &RightEdge );
		}
		if ( !TexStepEdge ( &LeftEdge ) ) {
			break;
		}
		if ( !TexStepEdge ( &RightEdge ) ) {
			break;
		}
		DestY++;
	}
}

