
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//	V.R. CONFERENCING APPLICATION                                       //
//                                                                          //
//	Written by R.M.J.Tucker                                             //
//                                                                          //
//	(C) Copyright 1996                                                  //
//                                                                          //
//	PCX.CPP                                                             //
//                                                                          //
//	Implementation of PCX class.                                        //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

#include "TEXTURE\PCX.H"

#include <alloc.h>
#include <io.h>
#include <fcntl.h>
#include <stdio.h>
#include <dos.h>

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Constructor to create PCX and do nothing else.                           //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

PCX :: PCX ( void )

/*	PRE	:
	POST	:
*/

{

}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Constructor to load a pcx directly into the structure.                   //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

PCX :: PCX ( char far *filename )

/*	PRE	:	`filename' is a valid PCX file format
	POST	:	PCX `filename' is loaded into RAM
*/

{
	LoadPCX ( filename );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Function to load a PCX directly into a structure.                        //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void PCX :: LoadPCX ( char far *filename )

/*	PRE	:	TRUE
	POST	:	PCX `filename' is loaded into RAM
*/

{
	enum {
		BYTEMODE,
		RUNMODE
	};
	int mode = BYTEMODE, bytecount;
	PIXEL abyte, *p;
	unsigned long Size, i;
	FILE *f;

	f = fopen ( ( const char * ) filename, "rb" );
	if ( f == NULL )
		return;
	fread ( &PCXHEADER, sizeof PCXHEADER, 1, f );

	Width = 1 + ( PCXHEADER.xmax - PCXHEADER.xmin );
	Length = 1 + ( PCXHEADER.ymax - PCXHEADER.ymin );
	Size = ( unsigned long ) Width * ( unsigned long ) Length;
	Bitmap = ( PIXEL *) farmalloc ( Size );
	if ( Bitmap == NULL )
		return;

	p = Bitmap;
	for ( i=0; i < Size; i++ ) {
		if ( mode == BYTEMODE ) {
			abyte = fgetc ( f );
			if ( abyte > 0xBF ) {
				bytecount = ( abyte & 0x3F );
				abyte = fgetc ( f );
				if ( --bytecount > 0 ) {
					mode = RUNMODE;
				}
			}
		} else {
			if  (  --bytecount == 0 ) {
				mode = BYTEMODE;
			}
		}
		*p++ = abyte;
	}

	/*	Load in the
		palette	*/

	fseek ( f, -768, SEEK_END );
	fread ( ( void * ) Palette, 768, 1, f );
	fclose ( f );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Operation to free the PCX memory and destroy the PCX.                    //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

PCX :: ~PCX ( void )

{
	if ( Bitmap != NULL ) {
		free ( Bitmap );
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Return the RGB colour palette for this image.                            //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

RGB * PCX :: GetPalette ( void )

/*	PRE	:
	POST	:
*/

{
	return ( Palette );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Set the VGA palette to that stored with this image.                      //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void PCX :: SetPalette ( void )

/*	PRE	:	PCX is a currently loaded PCX
	POST	:	Set the graphics palette to the PCX palette
*/

{
	union REGS regset;
	struct SREGS sregset;

	regset.x.ax = 0x1012;
	regset.x.bx = 0;
	regset.x.cx = 256;
	regset.x.dx = FP_OFF ( Palette );
	sregset.es = FP_SEG ( Palette );
	int86x ( 0x10, &regset, &regset, &sregset );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Return a pointer to the pixel image for this bitmap.                     //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

PIXEL * PCX :: GetPixels ( void )

/*	PRE	:	TRUE
	POST	:	Returns a pointer to the PCX BITMAP structure
*/

{
	return ( Bitmap );
}
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Return the width of the image.                                           //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

int PCX :: GetWidth ( void )

/*	PRE	:
	POST	:
*/

{
	return ( Width );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Return the length of the image.                                          //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

int PCX :: GetLength ( void )

/*	PRE	:
	POST	:
*/

{
	return ( Length );
}

void PCX :: Check ( void )

{
	int Max = -1000, Min = 1000;
	unsigned j;

	for ( j = 0; j < ( unsigned ) Width * ( unsigned ) Length; j++ ) {
		if ( Bitmap [j] > Max ) {
			Max = Bitmap [j];
		}
		if ( Bitmap [j] < Min ) {
			Min = Bitmap [j];
		}
	}

	printf ( "Colour depth is : %d --> %d = %d", Min, Max, ( Max - Min ) );
	return;
}