
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//	V.R. CONFERENCING APPLICATION                                       //
//                                                                          //
//	Written by R.M.J.Tucker                                             //
//                                                                          //
//	(C) Copyright 1996                                                  //
//                                                                          //
//	PCX.CPP                                                             //
//                                                                          //
//	Definition of PCX class.                                            //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

#if !defined ( __PCX__ )
	#define __PCX__

	#include "GLOBAL\TYPES.H"

	class PCX {

		private :

			/*	Structure equivalent
				to the PCX header.	*/

			struct {
				char manufacturer;     /* Always set to 0 */
				char version;	       /* Always 5 for 256-color files */
				char encoding;	       /* Always set to 1 */
				char bits_per_pixel;   /* Should be 8 for 256-color files */
				int  xmin, ymin;       /* Coordinates for top left corner */
				int  xmax, ymax;       /* Width and height of image */
				int  hres;             /* Horizontal resolution of image */
				int  vres;             /* Vertical resolution of image */
				char palette16 [ 48 ]; /* EGA palette; not used for 256-color files */
				char reserved;         /* Reserved for future use */
				char color_planes;     /* Color planes */
				int  bytes_per_line;   /* Number of bytes in 1 line of pixels */
				int  palette_type;     /* Should be 2 for color palette */
				char filler [ 58 ];    /* Nothing but junk */
			} PCXHEADER;

			/*	The colour palette
				for the image.	*/

			typedef struct {

				PIXEL Red, Green, Blue;

			} RGB;

			RGB	Palette [ 256 ];

			/*	The width and the
				length of the image.	*/

			int Width, Length;

			/*	Pixels pointer
				to the image.	*/

			PIXEL *Bitmap;

		public :

			/*	Create structure,
				and do nothing.	*/

			PCX ( void );

			/*	load a pcx
				image into RAM	*/

			PCX ( char far *file );

			/*	And explicit  load
				when already active	*/

			void LoadPCX ( char far *filename );

			/*	Destroy the PCX and
				free it's memory.	*/

			~PCX ();

			/*	Return the RAW
				bitmap for the PCX.	*/

			PIXEL *GetPixels ( void );

			/*	Return the palette
				entries for this bitmap.	*/

			RGB *GetPalette ( void );

			/*	Return the width
				of the image.	*/

			int GetWidth ( void );

			/*	Return the length
				of the image.	*/

			int GetLength ( void );

			/*	Setup the palette as
				you would for this image	*/

			void SetPalette ( void );
			void Check ( void );
    };
#endif
