
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//	V.R. CONFERENCING APPLICATION                                       //
//                                                                          //
//	Written by R.M.J.Tucker                                             //
//                                                                          //
//	(C) Copyright 1996                                                  //
//                                                                          //
//	VIEW.H                                                              //
//                                                                          //
//	Definition of the VIEWPROJECTION class.                             //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

#if !defined ( __VIEWPROJECTION__ )
	#define __VIEWPROJECTION__

	#include "CAMERA\CAMERA.H"
	#include "GEOMETRY\POLYGON.H"
	#include "GEOMETRY\GEOMETRY.H"
	#include "MODE\MODE.H"
	#include "MATH\PLANE.H"
	#include "VIEW\WINDOW.H"

	class VIEWPROJECTION {

		private :

			struct BUCKETPOLYGON {

				/*	The original polygon
					structure.	*/

				POLYGON *p;

				/*	A colour value for the
					( inside or outside ) polygon.	*/

				PIXEL Colour;

				/*	And the current texture
					map for this polygon.	*/

				TEXTUREMAP *Texture;

				/*	And the co-ordinates
					for the texture map.	*/

				INTPOINT *TexVerts;

				/*	and a base vertex in
					the bucket to work from.	*/

				int vBase;

				/*	Also state if the polygon
					was culled during transform.		*/

				BOOL Culled;
			};

			/*	Current number of vertices
				and polygons.	*/

			int nVertices, nPolygons;

			/*	The global 
				camera.	*/

			CAMERA *cWorld;

			/*	And the global
				light source.	*/

			CAMERA *cLight;

			/*	and the frustum
				planes.	*/

			#define NUMFRUSTUMPLANES 4

			PLANE Frustum [NUMFRUSTUMPLANES];

			/*	Projection
				parameters.	*/

			REAL xScale, yScale, maxScale, FieldOfView, xCenter, yCenter;

			/*	and the origin
				of the window.	*/

			WINDOW *Window;

		public :

			/*	Constructor sets default
				conditions.	*/

			VIEWPROJECTION ( void );

			/*	Add an object to
				the view projection.	*/

			VIEWPROJECTION& operator += ( GEOMETRY& g );

			/*	Display the contents
				so far.	*/

			void Show ( GRAPHICSMODE& Buffer );

			/*	Set the current 
				transform camera.	*/

			void SetCamera ( CAMERA *c );

			/*	Set the current
				light source.	*/

			void SetLight ( CAMERA *c );

			/*	Reset the viewing
				transformation.	*/

			void Reset ( WINDOW& Window );

			/*	Clip a polygon to the
				frustum and render it.	*/

			void ClipShadeAndRender	( GRAPHICSMODE& Buffer, BUCKETPOLYGON *Polygon, POINT *Vertices );

			/*	Setup the 6 plane
				view frustum.	*/

			void SetupFrustum ( WINDOW& Window );

			/*	Project a polygon to the
				current frustum parameters.	*/

			void Project ( int n, POINT *In, INTPOINT *Out );

			/*	Clip a polygon to 
				the view frustum.	*/

			BOOL ClipToViewFrustum ( int *nVerts, POINT *In, POINT *Out, INTPOINT *InTex, INTPOINT *OutTex );

			/*	Find the centroid
				of a polygon.	*/

			REAL GetDistance ( int Index );

			/*	Return the camera
				viewer.	*/

			CAMERA *GetCamera ( void );

			/*	Return the light
				source to the viewer.	*/

			CAMERA *GetLightSource ( void );

			/*	Return the current
				window to the viewer.	*/

			WINDOW *GetWindow ( void );
	};
#endif