
//////////////////////////////////////////////////////////////////////////////
//                                                                          //
//	V.R. CONFERENCING APPLICATION                                       //
//                                                                          //
//	Written by R.M.J.Tucker                                             //
//                                                                          //
//	(C) Copyright 1996                                                  //
//                                                                          //
//	MODETEX.CPP                                                         //
//                                                                          //
//	Implementation of MODE class.                                       //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////
	

#include "MODE\MODE.H"

#include <conio.h>
#include <stdio.h>
#include <math.h>
#include <dos.h>

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// This describes the clipping window for the purposes of Abrash's funcs.   //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

int ClipMinX = 0, ClipMinY= 0, ClipMaxX = 399, ClipMaxY = 319;

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// These are statics to avoid excessive parameter passing on the stack.     //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

static int	MaxVert, DestY, nVertices, TexMapDeltaY;
static PIXEL	FlatColour;
static INTPOINT	*VertexPtr;

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// This function is Abrash's edge stepper for the edges of the polygon.     //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

int GRAPHICSMODE :: FlatStepEdge ( EDGESCAN * Edge )

/*	PRE	:
	POST	:
*/

{
	if ( --Edge->RemainingScans == 0 ) {
		if ( FlatSetUpEdge ( Edge, Edge->CurrentEnd ) == 0 ) {
			return ( 0 );
		}
		return ( 1 );
	}

	Edge->DestX += Edge->DestXIntStep;  /* whole pixel step */

	if ( ( Edge->DestXErrTerm += Edge->DestXAdjUp ) > 0 ) {
		Edge->DestX += Edge->DestXDirection;
		Edge->DestXErrTerm -= Edge->DestXAdjDown;
	}

	return ( 1 );
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// This function is Abrash's edge stepper setup for polygon edges.          //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

int GRAPHICSMODE :: FlatSetUpEdge ( EDGESCAN * Edge, int StartVert )

/*	PRE	:
	POST	:
*/

{
	int	NextVert, DestXWidth;
	long	DestYHeight;

	for (;;) {
		if ( StartVert == MaxVert ) {
			return ( 0 );
		}

		NextVert = StartVert + Edge->Direction;
		if ( NextVert >= nVertices ) {
			NextVert = 0;
		} else if ( NextVert < 0 ) {
			NextVert = nVertices - 1;
		}

		if ( ( Edge->RemainingScans = VertexPtr [NextVert].y - VertexPtr [StartVert].y) != 0 ) {
			DestYHeight = INT_TO_FIXED ( Edge->RemainingScans );
			Edge->CurrentEnd = NextVert;
			Edge->DestX = VertexPtr [StartVert].x;
			if ( ( DestXWidth = ( VertexPtr [NextVert].x - VertexPtr [StartVert].x ) ) < 0 ) {
				Edge->DestXDirection = -1;
				DestXWidth = -DestXWidth;
				Edge->DestXErrTerm = 1 - Edge->RemainingScans;
				Edge->DestXIntStep = -( DestXWidth / Edge->RemainingScans );
			} else {
				Edge->DestXDirection = 1;
				Edge->DestXErrTerm = 0;
				Edge->DestXIntStep = DestXWidth / Edge->RemainingScans;
			}
			Edge->DestXAdjUp = DestXWidth % Edge->RemainingScans;
			Edge->DestXAdjDown = Edge->RemainingScans;
			return ( 1 );
		}
		StartVert = NextVert;
	}
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Abrash's outline scanner for the stepped edges of a polygon.             //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void GRAPHICSMODE :: FlatScanOutLine ( EDGESCAN * LeftEdge, EDGESCAN * RightEdge )

/*	PRE	:
	POST	:
*/

{
	int	DestX = LeftEdge->DestX, DestXMax = RightEdge->DestX;
	long	DestWidth;

	if ( ( DestXMax <= ClipMinX ) || ( DestX >= ClipMaxX ) ) {
		return;
	}

	if ( ( DestXMax - DestX ) <= 0 ) {
		return;
	}

	DestWidth = INT_TO_FIXED ( DestXMax - DestX );
	if ( DestXMax > ClipMaxX ) {
		DestXMax = ClipMaxX;
	}

	if ( DestX < ClipMinX ) {
		DestX = ClipMinX;
	}

	for (; DestX < DestXMax; DestX++ )
}

//////////////////////////////////////////////////////////////////////////////
//                                                                          //
// Render a texture mapped polygon onto the screen.                         //
//                                                                          //
//////////////////////////////////////////////////////////////////////////////

void GRAPHICSMODE :: RenderPolygon ( int nVerts, INTPOINT *Verts, INTPOINT *tVerts, TEXTUREMAP *Texture, BOOL Culled, PIXEL Colour )

/*	PRE	:	TRUE
	POST	:	Render textured polygon with the given colour.
*/

{
	int		MinY, MaxY, MinVert, i, j;
	EDGESCAN	LeftEdge, RightEdge;
	static		INTPOINT MyVerts [64];

	FlatColour = Colour;
	nVertices = nVerts;
	VertexPtr = Verts;

	if (nVertices < 3) {
		return;
	}

	if ( Culled == TRUE ) {

		/*	Culled, so vertices
			are in reverse order.	*/

		for ( j = 0, i = ( nVertices - 1 ); i > -1; i--, j++ ) {
			MyVerts [i].x = VertexPtr[j].y;
			MyVerts [i].y = VertexPtr[j].x;
		}
	} else {

		/*	Was not culled so vertices
			remain in the same order.	*/

		for ( i = 0; i < nVertices; i++ ) {
			MyVerts [i].x = VertexPtr[i].y;
			MyVerts [i].y = VertexPtr[i].x;
		}
	}

	VertexPtr = MyVerts;

	MinY = 32767;
	MaxY = -32767;
	for (i = 0; i < nVertices; i++ ) {
		if ( VertexPtr [i].y < MinY ) {
			MinY = VertexPtr [i].y;
			MinVert = i;
		}
		if ( VertexPtr [i].y > MaxY ) {
			MaxY = VertexPtr [i].y;
			MaxVert = i;
		}
	}

	if ( MinY >= MaxY ) {
		return;
	}
	DestY = MinY;

	LeftEdge.Direction = -1;
	FlatSetUpEdge ( &LeftEdge, MinVert );
	RightEdge.Direction = 1;
	FlatSetUpEdge ( &RightEdge, MinVert );

	for (;;) {
		if ( DestY >= ClipMaxY ) {
			return;
		}
		if ( DestY >= ClipMinY ) {
			FlatScanOutLine ( &LeftEdge, &RightEdge );
		}
		if ( !FlatStepEdge ( &LeftEdge ) ) {
			break;
		}
		if ( !FlatStepEdge ( &RightEdge ) ) {
			break;
		}
		DestY++;
	}
}
